@@ -26,14 +26,14 @@ Textfield Element.
 
 =head1 METHODS
 
-=head2 $self->containerize( $widget, $value, $errors )
+=head2 containerize
 
 =cut
 
 sub containerize {
-    my ( $self, $w, $value, $errors ) = @_;
+    my ( $self, $w, $value, $errors, $args ) = @_;
 
-    $value ||= $self->value;
+    $value = $self->value if ( not defined $value ) and not $args->{submitted};
 
     $value = ref $value eq 'ARRAY' ? shift @$value : $value;
 
@@ -44,6 +44,10 @@ sub containerize {
     return $self->container( { element => $i, error => $e, label => $l } );
 }
 
+=head1 SEE ALSO
+
+L<HTML::Widget::Element>
+
 =head1 AUTHOR
 
 Sebastian Riedel, C<sri@oook.de>
